/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.xwiki.officeimporter.internal.cleaner;

import java.io.StringReader;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.junit.Assert;
import org.junit.Test;

/**
 * Test case for cleaning html links ({@code<a/>} elements) in {@link OpenOfficeHTMLCleaner}.
 * 
 * @version $Id: e987bfc159c3a7ecdbdc8ab0f1c192ac5bd9f00e $
 * @since 1.8
 */
public class LinkOpenOfficeCleaningTest extends AbstractHTMLCleaningTest
{
    /**
     * The html generated by open office server includes anchors of the form {@code<a name="table1"><h1>Sheet 2:
     * <em>Hello</em></h1></a>} and the default html cleaner converts them to {@code <a name="table1"/><h1><a
     * name="table1">Sheet 1: <em>Hello</em></a></h1>} this is because of the close-before-copy-inside
     * behaviour of default html cleaner. Thus the additional (copy-inside) anchor needs to be ripped off.
     */
    @Test
    public void testDuplicateAnchorRemoving()
    {
        String html = header + "<a name=\"table1\"/><h1><a name=\"table1\">Sheet 1: <em>Hello</em></a></h1>" + footer;
        Document doc = openOfficeHTMLCleaner.clean(new StringReader(html));
        NodeList nodes = doc.getElementsByTagName("a");
        Assert.assertEquals(1, nodes.getLength());
        Element parent = (Element) nodes.item(0).getParentNode();
        Assert.assertEquals("p", parent.getNodeName());
    }

    /**
     * Test duplicate anchor filtering with TOC structures. see: http://jira.xwiki.org/jira/browse/XWIKI-3415
     */
    @Test
    public void testAnchorFilteringWithTOC() throws ClassCastException
    {
        String html = header + "<div>some text<h1><a name=\"Topic1\"/>Topic1</h1></div>" + footer;
        Document doc = openOfficeHTMLCleaner.clean(new StringReader(html));
        NodeList nodes = doc.getElementsByTagName("a");
        Assert.assertEquals(1, nodes.getLength());
    }

    /**
     * Test wrapping of html anchors with xwiki specific xhtml elements so that XHTML parser can recognize them.
     */
    @Test
    public void testAnchorWrapping()
    {
        String html = header + "<a name=\"name\"/>" + footer;
        Document doc = openOfficeHTMLCleaner.clean(new StringReader(html));
        NodeList nodes = doc.getElementsByTagName("a");
        Assert.assertEquals(1, nodes.getLength());
        Node anchor = nodes.item(0);
        Node beforeComment = anchor.getPreviousSibling();
        Assert.assertEquals(Node.COMMENT_NODE, beforeComment.getNodeType());
        Assert.assertEquals("startmacro:id|-|name=\"name\"|-|", beforeComment.getNodeValue());
        Node afterComment = anchor.getNextSibling();
        Assert.assertEquals(Node.COMMENT_NODE, afterComment.getNodeType());
        Assert.assertEquals("stopmacro", afterComment.getNodeValue());
    }
}
